%letter_test1
clear all;

% Modify the parameters below to specify how many test runs to do
% and how much noise to add, i.e. how many neurons to 'flip'

num_test_runs   =   1; 10;
num_flips       =   2;

%load data
D = [1 0 0 0 1 1 0 1 1 0 1 0 1 1 0 1 0 1 1 0 1 0 0 0 1];
X = reshape(D, 5, 5);
subplot(6,2,1);
imshow(X');
title('First Memory');
J = [0 0 0 0 0 1 1 1 0 1 1 1 1 0 1 0 1 1 0 1 0 0 0 1 1];
X = reshape(J, 5, 5);
subplot(6,2,2);
imshow(X');
title('Second Memory');
C = [1 0 0 0 0 0 1 1 1 1 0 1 1 1 1 0 1 1 1 1 1 0 0 0 0];
X = reshape(C, 5, 5);
subplot(6,2,3);
imshow(X');
title('Third Memory');
M = [0 1 1 1 0 0 0 1 0 0 0 1 0 1 0 0 1 1 1 0 0 1 1 1 0];
X = reshape(M, 5, 5);
subplot(6,2,4);
imshow(X');
title('Fourth Memory');
set(gcf,'Position', [10 50 400 300])
%create data matrix
data(1,:) = D;
data(2,:) = J;
data(3,:) = C;
data(4,:) = M;

n = 25; %number of data points in each picture
%make all 0s in data into -1s
datamones = data - ones(size(data));
newdata = data + datamones;

%make weights matrix (n x n)
w = zeros(n,n);
for i = 1:n
    for j = 1:n
        if i == j
            w(i,j) = 0; 
        else
            w(i,j) = newdata(:,i)'*newdata(:,j);
        end
    end
end

num_memories = size(newdata,1);

home; 
for run_num = 1:num_test_runs,
    %initial state
    Xinit = newdata(randi(num_memories),:);
    
    %initial state (with errors)
    temp  = ones(1,n);
    which_to_flip   =   randperm(n,num_flips);
    temp(which_to_flip) = -1;
    Xerr  = Xinit.*temp;
    
    %stable state
    Xerror = Xerr;
    Xtest = Xerror;
    X_new = Xtest;
    X_old = ones(size(Xtest));
    count = 0;
    while X_old*X_new'~=length(Xtest)
        count = count +1;
        % step 3: Asynchonous update
        if count == 50
            break
        else
            X_old = X_new; % previous new is now old
            X_new = AUpdate(X_old,w);
        end
    end
    
    %plot initial state, error state, and the stable state it converges to
    X = reshape(Xinit, 5, 5);
    subplot(3,1,1);
    imshow(X'); axis off;
    title('Initial State', 'FontSize', 20);
    X = reshape(Xerr, 5, 5);
    subplot(3,1,2);
    imshow(X'); axis off;
    title('Error State', 'FontSize', 20);
    X = reshape(X_new, 5, 5);
    subplot(3,1,3);
    imshow(X'); axis off;
    title('Stable State', 'FontSize', 20);
    %set(gcf,'Position',[ 1   727   546   214])
    
    pause;
    
    match_level(run_num) = (Xinit*X_new')/n;
end;

average_per_match   = mean((match_level+1)/2*100);
num_exact_match = sum(match_level >= .999);
num_exact_opposite = sum(match_level <= -.999);

percent_match       =   num_exact_match/num_test_runs*100;
percent_opposite    =   num_exact_opposite/num_test_runs*100;

home;
display(sprintf('Number of flips: %1.0f, Number of runs: %1.0f', num_flips, num_test_runs));
display(sprintf('Average percent overlap of neurons: %1.1f',average_per_match));
display(sprintf('Percent exact match for all neurons: %1.1f',percent_match));
display(sprintf('Percent exact opposite for all neurons: %1.1f',percent_opposite));